package com.sleepycat.je;

import java.io.PrintStream;

import com.sleepycat.je.BtreeStats;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.DatabaseStats;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.dbi.DatabaseImpl;

public privileged aspect CollectDatabaseStatsAspect {
	
	public DatabaseStats DatabaseImpl.stat(StatsConfig config) throws DatabaseException {

		if (stats == null) {

			/*
			 * Called first time w/ FAST_STATS so just give them an empty one.
			 */
			stats = new BtreeStats();
		}

		if (!config.getFast()) {
			if (tree == null) {
				return new BtreeStats();
			}

			PrintStream out = config.getShowProgressStream();
			if (out == null) {
				out = System.err;
			}

			com.sleepycat.je.dbi.StatsAccumulator statsAcc = new com.sleepycat.je.dbi.StatsAccumulator(out, config
					.getShowProgressInterval(), getEmptyStats());
			walkDatabaseTree(statsAcc, out, true);
			statsAcc.copyToStats(stats);
		}

		return stats;
	}
	
	/* @return the right kind of stats object for this database. */
	public DatabaseStats DatabaseImpl.getEmptyStats() {
		return new BtreeStats();
	}
}
