package com.sleepycat.je;

import com.sleepycat.je.cleaner.CleanerStatsInfrastructure;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.dbi.LogBufferBudget;
import com.sleepycat.je.incomp.INCompressorStatsInfrastructure;
import com.sleepycat.je.log.FSyncStatsInfrastructure;
import com.sleepycat.je.log.FileManager;
import com.sleepycat.je.log.LogBufferPool;
import com.sleepycat.je.log.LogManager;

public privileged aspect EnvStatsInfrastructure {
	
	declare precedence : 
		INCompressorStatsInfrastructure, 
		EvictorStatsInfrastructure, 
		CheckpointerStatsInfrastructure, 
		CleanerStatsInfrastructure, 
		CachingStatsInfrastructure, 
		FSyncStatsInfrastructure, 
		LogStatsInfrastructure;

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public EnvironmentStats Environment.getStats(StatsConfig config)
			throws DatabaseException {

		StatsConfig useConfig = (config == null) ? StatsConfig.DEFAULT : config;

		if (environmentImpl != null) {
			return environmentImpl.loadStats(useConfig);
		} else {
			return new EnvironmentStats();
		}
	}

	/**
	 * Retrieve and return stat information.
	 */
	synchronized public EnvironmentStats EnvironmentImpl.loadStats(
			StatsConfig config) throws DatabaseException {

		EnvironmentStats stats = new EnvironmentStats();
		loadStats(config, stats);
		return stats;
	}

	void LogBufferBudget.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {
	}
	
	// should be protected
	void EnvironmentImpl.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {

		logManager.loadStats(config, stats);
		
	}


	void LogBufferPool.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {
	}

	public void LogManager.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {
		logBufferPool.loadStats(config, stats);
		fileManager.loadStats(config,stats);
	}

	void FileManager.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {
	}

}
