package com.sleepycat.je.log;

import java.util.Iterator;

import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;

public privileged aspect CollectLogBufferPoolStats  
	perthis(loadStats(StatsConfig,EnvironmentStats) || readBuffer()) {

	/* Stats LogBufferPool */
	private long nNotResident = 0; // had to be instantiated from an lsn

	private long nCacheMiss = 0; // had to retrieve from disk

	pointcut loadStats(StatsConfig config, EnvironmentStats stats):
			execution(void LogBufferPool.loadStats(StatsConfig, EnvironmentStats)) && 
			args(config,stats);

	after(StatsConfig config, EnvironmentStats stats, LogBufferPool lbp) : 
			loadStats(config, stats) && this(lbp) {

		stats.setNCacheMiss(nCacheMiss);
		stats.setNNotResident(nNotResident);
		if (config.getClear()) {
			nCacheMiss = 0;
			nNotResident = 0;
		}

		/* Also return buffer pool memory usage */
		// Lck bufferPoolLatch.acquire();
		long bufferBytes = 0;
		int nLogBuffers = 0;
		// try {
		Iterator iter = lbp.bufferPool.iterator();
		while (iter.hasNext()) {
			LogBuffer l = (LogBuffer) iter.next();
			nLogBuffers++;
			bufferBytes += l.getCapacity();
		}
		// } finally {
		// Lck bufferPoolLatch.release();
		// }
		stats.setNLogBuffers(nLogBuffers);
		stats.setBufferBytes(bufferBytes);
	}

	pointcut readBuffer():execution(LogBuffer LogBufferPool.getReadBuffer(long));

	LogBuffer around(LogBufferPool lbp):
			readBuffer() && this(lbp) {

		nNotResident++;
		LogBuffer foundBuffer = proceed(lbp);
		if (foundBuffer == null) {
			nCacheMiss++;
		}
		return foundBuffer;
	}

	after() returning (LogBuffer foundBuffer) : 
			execution(LogBuffer LogBufferPool.getReadBuffer(long)) 
					{
		if (foundBuffer == null)
			nCacheMiss++;
	}

}
