package com.sleepycat.je;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.recovery.Checkpointer;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.IN;

public privileged aspect CheckpointerStats {





	after(Checkpointer cp):execution(Checkpointer.new(..)) && target(cp){
		cp.nCheckpoints = 0;
	}

	after(Checkpointer cp):
		set(private long Checkpointer.checkpointId) && withincode(* Checkpointer.doCheckpoint(..)) && target(cp) {
		cp.nCheckpoints++;
	}
	before(Checkpointer cp):
		set(private long Checkpointer.checkpointId) && withincode(* Checkpointer.doCheckpoint(..)) && target(cp) {
		cp.resetPerRunCounters();
	}
	


	after(Checkpointer cp):call(* Checkpointer.hook_flushed(..)) && target(cp){
		cp.nFullINFlushThisRun++;
		cp.nFullINFlush++;
	}
	after(Checkpointer cp):call(* Checkpointer.hook_deltaINFlush(..)) && target(cp){
		cp.nDeltaINFlushThisRun++;
		cp.nDeltaINFlush++;
	}
	before(Checkpointer cp, IN targetIN):call(* Checkpointer.hook_updateParent(..)) && target(cp) && args(targetIN,..){
		cp.nFullINFlushThisRun++;
		cp.nFullINFlush++;
		if (targetIN instanceof BIN) {
			cp.nFullBINFlush++;
		}
	}	

	
	
	/* Stats */
	private int Checkpointer.nCheckpoints;

	private int Checkpointer.nFullINFlush;

	private int Checkpointer.nFullBINFlush;

	private int Checkpointer.nDeltaINFlush;

	private int Checkpointer.nFullINFlushThisRun;

	private int Checkpointer.nDeltaINFlushThisRun;

	/**
	 * Load stats.
	 */
	public void Checkpointer.loadStats(StatsConfig config, EnvironmentStats stat)
			throws DatabaseException {

		stat.setNCheckpoints(nCheckpoints);
		stat.setLastCheckpointStart(lastCheckpointStart);
		stat.setLastCheckpointEnd(lastCheckpointEnd);
		stat.setLastCheckpointId(checkpointId);
		stat.setNFullINFlush(nFullINFlush);
		stat.setNFullBINFlush(nFullBINFlush);
		stat.setNDeltaINFlush(nDeltaINFlush);

		if (config.getClear()) {
			nCheckpoints = 0;
			nFullINFlush = 0;
			nFullBINFlush = 0;
			nDeltaINFlush = 0;
		}
	}
	
	/**
	 * Reset per-run counters.
	 */
	private void Checkpointer.resetPerRunCounters() {
		nFullINFlushThisRun = 0;
		nDeltaINFlushThisRun = 0;
		/* nAlreadyEvictedThisRun = 0; -- for future */
	}
}
