package com.sleepycat.je;

import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.recovery.Checkpointer;
import com.sleepycat.je.utilint.DbLsn;

public privileged aspect CheckpointerStatsInfrastructure {
	
	/* Checkpointer */

	/**
	 * The total number of checkpoints run so far.
	 */
	private int EnvironmentStats.nCheckpoints;

	/**
	 * The Id of the last checkpoint.
	 */
	private long EnvironmentStats.lastCheckpointId;

	/**
	 * The accumulated number of full INs flushed to the log.
	 */
	private int EnvironmentStats.nFullINFlush;

	/**
	 * The accumulated number of full BINs flushed to the log.
	 */
	private int EnvironmentStats.nFullBINFlush;

	/**
	 * The accumulated number of Delta INs flushed to the log.
	 */
	private int EnvironmentStats.nDeltaINFlush;

	/**
	 * The location in the log of the last checkpoint start.
	 */
	private long EnvironmentStats.lastCheckpointStart;

	/**
	 * The location in the log of the last checkpoint end.
	 */
	private long EnvironmentStats.lastCheckpointEnd;
	
	
	after(EnvironmentStats envStats):
		execution(* EnvironmentStats.reset(..)) && target(envStats){
		// Checkpointer
		envStats.nCheckpoints = 0;
		envStats.lastCheckpointId = 0;
		envStats.nFullINFlush = 0;
		envStats.nFullBINFlush = 0;
		envStats.nDeltaINFlush = 0;
		envStats.lastCheckpointStart = DbLsn.NULL_LSN;
		envStats.lastCheckpointEnd = DbLsn.NULL_LSN;
	}
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getLastCheckpointId() {
		return lastCheckpointId;
	}



	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNCheckpoints() {
		return nCheckpoints;
	}
	
	
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNDeltaINFlush() {
		return nDeltaINFlush;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getLastCheckpointEnd() {
		return lastCheckpointEnd;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getLastCheckpointStart() {
		return lastCheckpointStart;
	}
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNFullINFlush() {
		return nFullINFlush;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNFullBINFlush() {
		return nFullBINFlush;
	}
	

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setLastCheckpointId(long l) {
		lastCheckpointId = l;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNCheckpoints(int val) {
		nCheckpoints = val;
	}
	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNDeltaINFlush(int val) {
		nDeltaINFlush = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setLastCheckpointEnd(long lsn) {
		lastCheckpointEnd = lsn;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setLastCheckpointStart(long lsn) {
		lastCheckpointStart = lsn;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNFullINFlush(int val) {
		nFullINFlush = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNFullBINFlush(int val) {
		nFullBINFlush = val;
	}
	
	
	//TODO precedence
	before(StringBuffer sb, EnvironmentStats envStats):
		execution(* EnvironmentStats.hook_fillToStringBuffer(..)) && args(sb) && target(envStats)
	{		// Checkpointer
		sb.append("nCheckpoints=").append(envStats.nCheckpoints).append('\n');
		sb.append("lastCheckpointId=").append(envStats.lastCheckpointId).append('\n');
		sb.append("nFullINFlush=").append(envStats.nFullINFlush).append('\n');
		sb.append("nFullBINFlush=").append(envStats.nFullBINFlush).append('\n');
		sb.append("nDeltaINFlush=").append(envStats.nDeltaINFlush).append('\n');
		sb.append("lastCheckpointStart=").append(
				DbLsn.getNoFormatString(envStats.lastCheckpointStart)).append('\n');
		sb.append("lastCheckpointEnd=").append(
				DbLsn.getNoFormatString(envStats.lastCheckpointEnd)).append('\n');
	}
	
	
	//TODO precedence
	before(StatsConfig config, EnvironmentStats stats, EnvironmentImpl envImpl) throws DatabaseException:
		execution(* EnvironmentImpl.loadStats(StatsConfig, EnvironmentStats)) && this(envImpl) && args(config, stats){
		
		envImpl.checkpointer.loadStats(config, stats);
	}
	
}
