package com.sleepycat.je.cleaner;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect CleanerStatsInfrastructure {

	/* Cleaner */

	/** The number of files to be cleaned to reach the target utilization. */
	private int EnvironmentStats.cleanerBacklog;

	/** The number of cleaner runs this session. */
	private int EnvironmentStats.nCleanerRuns;

	/** The number of cleaner file deletions this session. */
	private int EnvironmentStats.nCleanerDeletions;

	/**
	 * The accumulated number of INs obsolete.
	 */
	private int EnvironmentStats.nINsObsolete;

	/**
	 * The accumulated number of INs cleaned.
	 */
	private int EnvironmentStats.nINsCleaned;

	/**
	 * The accumulated number of INs that were not found in the tree anymore
	 * (deleted).
	 */
	private int EnvironmentStats.nINsDead;

	/**
	 * The accumulated number of INs migrated.
	 */
	private int EnvironmentStats.nINsMigrated;

	/**
	 * The accumulated number of LNs obsolete.
	 */
	private int EnvironmentStats.nLNsObsolete;

	/**
	 * The accumulated number of LNs cleaned.
	 */
	private int EnvironmentStats.nLNsCleaned;

	/**
	 * The accumulated number of LNs that were not found in the tree anymore
	 * (deleted).
	 */
	private int EnvironmentStats.nLNsDead;

	/**
	 * The accumulated number of LNs encountered that were locked.
	 */
	private int EnvironmentStats.nLNsLocked;

	/**
	 * The accumulated number of LNs encountered that were migrated forward in
	 * the log.
	 */
	private int EnvironmentStats.nLNsMigrated;

	/**
	 * The accumulated number of LNs that were marked for migration during
	 * cleaning.
	 */
	private int EnvironmentStats.nLNsMarked;

	/**
	 * The accumulated number of LNs processed without a tree lookup.
	 */
	private int EnvironmentStats.nLNQueueHits;

	/**
	 * The accumulated number of LNs processed because they were previously
	 * locked.
	 */
	private int EnvironmentStats.nPendingLNsProcessed;

	/**
	 * The accumulated number of LNs processed because they were previously
	 * marked for migration.
	 */
	private int EnvironmentStats.nMarkedLNsProcessed;

	/**
	 * The accumulated number of LNs processed because they are soon to be
	 * cleaned.
	 */
	private int EnvironmentStats.nToBeCleanedLNsProcessed;

	/**
	 * The accumulated number of LNs processed because they qualify for
	 * clustering.
	 */
	private int EnvironmentStats.nClusterLNsProcessed;

	/**
	 * The accumulated number of pending LNs that could not be locked for
	 * migration because of a long duration application lock.
	 */
	private int EnvironmentStats.nPendingLNsLocked;

	/**
	 * The accumulated number of log entries read by the cleaner.
	 */
	private int EnvironmentStats.nCleanerEntriesRead;





	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getCleanerBacklog() {
		return cleanerBacklog;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNCleanerRuns() {
		return nCleanerRuns;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNCleanerDeletions() {
		return nCleanerDeletions;
	}



	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNCleanerEntriesRead() {
		return nCleanerEntriesRead;
	}







	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNINsObsolete() {
		return nINsObsolete;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNINsCleaned() {
		return nINsCleaned;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNINsDead() {
		return nINsDead;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNINsMigrated() {
		return nINsMigrated;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsObsolete() {
		return nLNsObsolete;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsCleaned() {
		return nLNsCleaned;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsDead() {
		return nLNsDead;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsLocked() {
		return nLNsLocked;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsMigrated() {
		return nLNsMigrated;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNsMarked() {
		return nLNsMarked;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNLNQueueHits() {
		return nLNQueueHits;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNPendingLNsProcessed() {
		return nPendingLNsProcessed;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNMarkedLNsProcessed() {
		return nMarkedLNsProcessed;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNToBeCleanedLNsProcessed() {
		return nToBeCleanedLNsProcessed;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNClusterLNsProcessed() {
		return nClusterLNsProcessed;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNPendingLNsLocked() {
		return nPendingLNsLocked;
	}


	

	

	


	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setCleanerBacklog(int val) {
		cleanerBacklog = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNCleanerRuns(int val) {
		nCleanerRuns = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNCleanerDeletions(int val) {
		nCleanerDeletions = val;
	}
	

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNCleanerEntriesRead(int val) {
		nCleanerEntriesRead = val;
	}






	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNINsObsolete(int val) {
		nINsObsolete = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNINsCleaned(int val) {
		nINsCleaned = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNINsDead(int val) {
		nINsDead = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNINsMigrated(int val) {
		nINsMigrated = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsObsolete(int val) {
		nLNsObsolete = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsCleaned(int val) {
		nLNsCleaned = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsDead(int val) {
		nLNsDead = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsLocked(int val) {
		nLNsLocked = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsMigrated(int val) {
		nLNsMigrated = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNsMarked(int val) {
		nLNsMarked = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNLNQueueHits(int val) {
		nLNQueueHits = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNPendingLNsProcessed(int val) {
		nPendingLNsProcessed = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNMarkedLNsProcessed(int val) {
		nMarkedLNsProcessed = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNToBeCleanedLNsProcessed(int val) {
		nToBeCleanedLNsProcessed = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNClusterLNsProcessed(int val) {
		nClusterLNsProcessed = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNPendingLNsLocked(int val) {
		nPendingLNsLocked = val;
	}

	

	

	after(EnvironmentStats envStats):
		execution(* EnvironmentStats.reset(..)) && target(envStats){
		// Cleaner
		envStats.cleanerBacklog = 0;
		envStats.nCleanerRuns = 0;
		envStats.nCleanerDeletions = 0;
		envStats.nINsObsolete = 0;
		envStats.nINsCleaned = 0;
		envStats.nINsDead = 0;
		envStats.nINsMigrated = 0;
		envStats.nLNsObsolete = 0;
		envStats.nLNsCleaned = 0;
		envStats.nLNsDead = 0;
		envStats.nLNsLocked = 0;
		envStats.nLNsMigrated = 0;
		envStats.nLNsMarked = 0;
		envStats.nLNQueueHits = 0;
		envStats.nPendingLNsProcessed = 0;
		envStats.nMarkedLNsProcessed = 0;
		envStats.nToBeCleanedLNsProcessed = 0;
		envStats.nClusterLNsProcessed = 0;
		envStats.nPendingLNsLocked = 0;
		envStats.nCleanerEntriesRead = 0;
	}
	
	//TODO precedence
	before(StringBuffer sb, EnvironmentStats envStats):
		execution(* EnvironmentStats.hook_fillToStringBuffer(..)) && args(sb) && target(envStats)
	{		
	
		
		// Cleaner
		sb.append("cleanerBacklog=").append(envStats.cleanerBacklog).append('\n');
		sb.append("nCleanerRuns=").append(envStats.nCleanerRuns).append('\n');
		sb.append("nCleanerDeletions=").append(envStats.nCleanerDeletions).append('\n');
		sb.append("nINsObsolete=").append(envStats.nINsObsolete).append('\n');
		sb.append("nINsCleaned=").append(envStats.nINsCleaned).append('\n');
		sb.append("nINsDead=").append(envStats.nINsDead).append('\n');
		sb.append("nINsMigrated=").append(envStats.nINsMigrated).append('\n');
		sb.append("nLNsObsolete=").append(envStats.nLNsObsolete).append('\n');
		sb.append("nLNsCleaned=").append(envStats.nLNsCleaned).append('\n');
		sb.append("nLNsDead=").append(envStats.nLNsDead).append('\n');
		sb.append("nLNsLocked=").append(envStats.nLNsLocked).append('\n');
		sb.append("nLNsMigrated=").append(envStats.nLNsMigrated).append('\n');
		sb.append("nLNsMarked=").append(envStats.nLNsMarked).append('\n');
		sb.append("nLNQueueHits=").append(envStats.nLNQueueHits).append('\n');
		sb.append("nPendingLNsProcessed=").append(envStats.nPendingLNsProcessed).append(
				'\n');
		sb.append("nMarkedLNsProcessed=").append(envStats.nMarkedLNsProcessed).append(
				'\n');
		sb.append("nToBeCleanedLNsProcessed=").append(envStats.nToBeCleanedLNsProcessed)
				.append('\n');
		sb.append("nClusterLNsProcessed=").append(envStats.nClusterLNsProcessed).append(
				'\n');
		sb.append("nPendingLNsLocked=").append(envStats.nPendingLNsLocked).append('\n');
		sb.append("nCleanerEntriesRead=").append(envStats.nCleanerEntriesRead).append(
				'\n');

	}
	
	//TODO precedence
	before(StatsConfig config, EnvironmentStats stats, EnvironmentImpl envImpl) throws DatabaseException:
		execution(* EnvironmentImpl.loadStats(StatsConfig, EnvironmentStats)) && this(envImpl) && args(config, stats){
		
		envImpl.cleaner.loadStats(config, stats);
	}

}
