package com.sleepycat.je.cleaner;

import java.util.Map;

import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.log.CleanerFileReader;
import com.sleepycat.je.tree.DIN;
import com.sleepycat.je.tree.IN;
import com.sleepycat.je.tree.Tree;
import com.sleepycat.je.tree.TreeLocation;
import com.sleepycat.je.txn.LockGrantType;
import com.sleepycat.je.txn.LockResult;

public privileged aspect FileProcessorStats {

	/* Per Run counters. Reset before each file is processed. */
	private int FileProcessor.nINsObsoleteThisRun = 0;

	private int FileProcessor.nINsCleanedThisRun = 0;

	private int FileProcessor.nINsDeadThisRun = 0;

	private int FileProcessor.nINsMigratedThisRun = 0;

	private int FileProcessor.nLNsObsoleteThisRun = 0;

	private int FileProcessor.nLNsCleanedThisRun = 0;

	private int FileProcessor.nLNsDeadThisRun = 0;

	private int FileProcessor.nLNsLockedThisRun = 0;

	private int FileProcessor.nLNsMigratedThisRun = 0;

	private int FileProcessor.nLNsMarkedThisRun = 0;

	private int FileProcessor.nLNQueueHitsThisRun = 0;

	private int FileProcessor.nEntriesReadThisRun;

	/**
	 * Reset per-run counters.
	 */
	private void FileProcessor.resetPerRunCounters() {
		nINsObsoleteThisRun = 0;
		nINsCleanedThisRun = 0;
		nINsDeadThisRun = 0;
		nINsMigratedThisRun = 0;
		nLNsObsoleteThisRun = 0;
		nLNsCleanedThisRun = 0;
		nLNsDeadThisRun = 0;
		nLNsMigratedThisRun = 0;
		nLNsMarkedThisRun = 0;
		nLNQueueHitsThisRun = 0;
		nLNsLockedThisRun = 0;
		nEntriesReadThisRun = 0;
	}

	/**
	 * Add per-run counters to total counters.
	 */
	private void FileProcessor.accumulatePerRunCounters() {
		cleaner.nINsObsolete += nINsObsoleteThisRun;
		cleaner.nINsCleaned += nINsCleanedThisRun;
		cleaner.nINsDead += nINsDeadThisRun;
		cleaner.nINsMigrated += nINsMigratedThisRun;
		cleaner.nLNsObsolete += nLNsObsoleteThisRun;
		cleaner.nLNsCleaned += nLNsCleanedThisRun;
		cleaner.nLNsDead += nLNsDeadThisRun;
		cleaner.nLNsMigrated += nLNsMigratedThisRun;
		cleaner.nLNsMarked += nLNsMarkedThisRun;
		cleaner.nLNQueueHits += nLNQueueHitsThisRun;
		cleaner.nLNsLocked += nLNsLockedThisRun;
	}

	before(FileProcessor fp):call(* FileProcessor.hook_cleanFile(..))&&this(fp){
		fp.resetPerRunCounters();
		fp.cleaner.nCleanerRuns++;
	}

	after(FileProcessor fp) returning (boolean result):
		call(* FileProcessor.processFile(..)) && 
		withincode(* FileProcessor.hook_cleanFile(..)) && this(fp){

		if (result)
			fp.accumulatePerRunCounters();
	}

	String around(FileProcessor fp):
		execution(* FileProcessor.getEndTraceMenssage(..)) && this(fp) {

		return proceed(fp) + " nEntriesRead=" + fp.nEntriesReadThisRun
				+ " nINsObsolete=" + fp.nINsObsoleteThisRun + " nINsCleaned="
				+ fp.nINsCleanedThisRun + " nINsDead=" + fp.nINsDeadThisRun
				+ " nINsMigrated=" + fp.nINsMigratedThisRun + " nLNsObsolete="
				+ fp.nLNsObsoleteThisRun + " nLNsCleaned="
				+ fp.nLNsCleanedThisRun + " nLNsDead=" + fp.nLNsDeadThisRun
				+ " nLNsMigrated=" + fp.nLNsMigratedThisRun + " nLNsMarked="
				+ fp.nLNsMarkedThisRun + " nLNQueueHits="
				+ fp.nLNQueueHitsThisRun + " nLNsLocked="
				+ fp.nLNsLockedThisRun;
	}

	String around(FileProcessor fp):
		execution(* FileProcessor.hook_getBeginTraceMenssage(..)) && this(fp){

		return proceed(fp) + " backlog=" + fp.cleaner.nBacklogFiles;
	}

	after(FileProcessor fp) returning (boolean result):
		call(* readNextEntry(..)) && withincode(* FileProcessor.hookr_processFileInternalLoop(..)) && this(fp){

		if (result)
			fp.cleaner.nEntriesRead += 1;
	}

	before(boolean isLN, boolean isIN, FileProcessor fp):
		call(* FileProcessor.hook_obsoleteRun(..)) && args(isLN, isIN)&& this(fp){

		/* Count obsolete stats. */
		if (isLN) {
			fp.nLNsObsoleteThisRun++;
		} else if (isIN) {
			fp.nINsObsoleteThisRun++;
		}
	}

	before(FileProcessor fp, CleanerFileReader reader):
		execution(* FileProcessor.hook_finishedReading(..))&&args(reader)&& this(fp){

		fp.nEntriesReadThisRun = reader.getNumRead();
	}

	before(FileProcessor fp):execution(* FileProcessor.processLN(..))&& this(fp){
		fp.nLNsCleanedThisRun++;
	}

	after(FileProcessor fp) returning (boolean r):
		call(boolean Cleaner.hook_checkDeletedDb(DatabaseImpl)) && this(fp)
		&& withincode(void FileProcessor.processLN(Long, TreeLocation, Long,LNInfo, Map))
	{

		if (r)
			fp.nLNsDeadThisRun++;
	}

	after(FileProcessor fp) returning (boolean parentFound): call(* Tree.getParentBINForChildLN(..)) && withincode(* FileProcessor.processLN(..))&& this(fp){
		if (!parentFound)
			fp.nLNsDeadThisRun++;
	}

	after(FileProcessor fp): call(* FileProcessor.hook_entryIsKnownDeleted(..)) && this(fp){
		fp.nLNsDeadThisRun++;
	}

	before(FileProcessor fp, DIN parentDIN):call(* FileProcessor.processFoundLN(..))  && withincode(* FileProcessor.processLN(..))&&args(..,parentDIN)&& this(fp){
		if (parentDIN == null) {
			fp.nLNQueueHitsThisRun++;
			fp.nLNsCleanedThisRun++;
		}
	}

	after(FileProcessor fp) returning(LockResult lockRet):
		call(* nonBlockingLock(..)) && withincode(* FileProcessor.processFoundLN(..))&& this(fp){
		if (lockRet.getLockGrant() == LockGrantType.DENIED) {
			fp.nLNsLockedThisRun++;
		} else {
			fp.nLNsDeadThisRun++;
		}
	}

	after(FileProcessor fp):call(* FileProcessor.hook_lnsMarked(..))&& this(fp){
		fp.nLNsMarkedThisRun++;
	}

	before(FileProcessor fp):execution(* FileProcessor.processIN(..))&& this(fp){
		fp.nINsCleanedThisRun++;
	}

	after(FileProcessor fp) returning (IN inInTree):
		call(* FileProcessor.findINInTree(..)) && withincode(* FileProcessor.processIN(..))&& this(fp){

		if (inInTree == null) {
			fp.nINsDeadThisRun++;
		} else {
			fp.nINsMigratedThisRun++;
		}
	}

}
