package com.sleepycat.je;

import com.sleepycat.je.evictor.Evictor;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.IN;

public privileged aspect EvictorStats perthis(initEvictor()) {
	pointcut initEvictor():execution(Evictor.new(..));
	
	/*
	 * Stats
	 */

	/* Number of passes made to the evictor. */
	private int nEvictPasses = 0;

	/* Number of nodes selected to evict. */
	private long nNodesSelected = 0;

	private long nNodesSelectedThisRun;

	/* Number of nodes scanned in order to select the eviction set */
	private int nNodesScanned = 0;


	/*
	 * Number of nodes evicted on this run. This could be understated, as a
	 * whole subtree may have gone out with a single node.
	 */
	private long nNodesEvicted = 0;

	private long nNodesEvictedThisRun;

	/* Number of BINs stripped. */
	private long nBINsStripped = 0;

	private long nBINsStrippedThisRun;
	
	
	/**
	 * Load stats.
	 */
	public void Evictor.loadStats(StatsConfig config, EnvironmentStats stat)
			throws DatabaseException {
		stat.setRequiredEvictBytes(currentRequiredEvictBytes);
	}

	after(StatsConfig config, EnvironmentStats stat) :
		execution(* Evictor.loadStats(..)) && args(config, stat) {
		
		stat.setNEvictPasses(nEvictPasses);
		stat.setNNodesSelected(nNodesSelected);
		stat.setNNodesScanned(nNodesScanned);
		stat.setNNodesExplicitlyEvicted(nNodesEvicted);
		stat.setNBINsStripped(nBINsStripped);

		if (config.getClear()) {
			nEvictPasses = 0;
			nNodesSelected = 0;
			nNodesScanned = 0;
			nNodesEvicted = 0;
			nBINsStripped = 0;
		}
	}	
	
	before(Evictor evictor):execution(* Evictor.evictBatch(..)) && this(evictor){
		nNodesSelectedThisRun = 0;
		nNodesEvictedThisRun = 0;
		evictor.nNodesScannedThisRun = 0;
		nBINsStrippedThisRun = 0;
		nEvictPasses++;
	}
	
	long around(Evictor evictor):
		execution(* Evictor.hook_evictInternal(..)) && this(evictor){
		
		long l=0;
		try{
			l=proceed(evictor);
		} finally{
			nNodesScanned += evictor.nNodesScannedThisRun;
		}
		return l;
	}
	
	after() returning (IN targetIN):execution(* Evictor.selectIN(..)) {
		if (targetIN != null) {
			nNodesSelectedThisRun++;
			nNodesSelected++;
		}
	}
	
	after() returning (long evictedBytes):
		call(* BIN.evictLNs(..)) && within(Evictor) {
		if (evictedBytes > 0) {
			nBINsStrippedThisRun++;
			nBINsStripped++;
		}
	}
	
	after():call(* Evictor.hook_evictedIN(..)){
		/* Stats */
		nNodesEvictedThisRun++;
		nNodesEvicted++;
	}
	
}
