package com.sleepycat.je;

import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect EvictorStatsInfrastructure {
	
	
	/* Evictor */

	/**
	 * The number of passes made to the evictor.
	 */
	private int EnvironmentStats.nEvictPasses;

	/**
	 * The accumulated number of nodes selected to evict.
	 */
	private long EnvironmentStats.nNodesSelected;

	/**
	 * The accumulated number of nodes scanned in order to select the eviction
	 * set.
	 */
	private long EnvironmentStats.nNodesScanned;

	/**
	 * The accumulated number of nodes evicted.
	 */
	private long EnvironmentStats.nNodesExplicitlyEvicted;

	/**
	 * The number of BINs stripped by the evictor.
	 */
	private long EnvironmentStats.nBINsStripped;

	/**
	 * The number of bytes we need to evict in order to get under budget.
	 */
	private long EnvironmentStats.requiredEvictBytes;

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNEvictPasses() {
		return nEvictPasses;
	}
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNNodesExplicitlyEvicted() {
		return nNodesExplicitlyEvicted;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNBINsStripped() {
		return nBINsStripped;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getRequiredEvictBytes() {
		return requiredEvictBytes;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNNodesScanned() {
		return nNodesScanned;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNNodesSelected() {
		return nNodesSelected;
	}
	
	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNEvictPasses(int val) {
		nEvictPasses = val;
	}
	
	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNNodesExplicitlyEvicted(long l) {
		nNodesExplicitlyEvicted = l;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setRequiredEvictBytes(long l) {
		requiredEvictBytes = l;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNBINsStripped(long l) {
		nBINsStripped = l;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNNodesScanned(long l) {
		nNodesScanned = l;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNNodesSelected(long l) {
		nNodesSelected = l;
	}
	
	after(EnvironmentStats envStats):
		execution(* EnvironmentStats.reset(..)) && target(envStats){
		// Evictor
		envStats.nEvictPasses = 0;
		envStats.nNodesSelected = 0;
		envStats.nNodesScanned = 0;
		envStats.nNodesExplicitlyEvicted = 0;
		envStats.nBINsStripped = 0;
		envStats.requiredEvictBytes = 0;
	}
	
	//TODO precedence
	before(StringBuffer sb, EnvironmentStats envStats):
		execution(* EnvironmentStats.hook_fillToStringBuffer(..)) && args(sb) && target(envStats)
	{		// Checkpointer
		// Evictor
		sb.append("nEvictPasses=").append(envStats.nEvictPasses).append('\n');
		sb.append("nNodesSelected=").append(envStats.nNodesSelected).append('\n');
		sb.append("nNodesScanned=").append(envStats.nNodesScanned).append('\n');
		sb.append("nNodesExplicitlyEvicted=").append(envStats.nNodesExplicitlyEvicted)
				.append('\n');
		sb.append("nBINsStripped=").append(envStats.nBINsStripped).append('\n');
		sb.append("requiredEvictBytes=").append(envStats.requiredEvictBytes)
				.append('\n');

	}
	
	//TODO precedence
	before(StatsConfig config, EnvironmentStats stat, EnvironmentImpl envImpl) throws DatabaseException:
		execution(* EnvironmentImpl.loadStats(StatsConfig, EnvironmentStats)) && this(envImpl) && args(config, stat){
		
		envImpl.evictor.loadStats(config, stat);
	}
}
