package com.sleepycat.je.log;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;

public privileged aspect FSyncStats {
	

	/* stats */
	private long FSyncManager.nFSyncRequests = 0;

	private long FSyncManager.nFSyncs = 0;

	private long FSyncManager.nTimeouts = 0;
	
	long FSyncManager.getNFSyncRequests() {
		return nFSyncRequests;
	}

	long FSyncManager.getNFSyncs() {
		return nFSyncs;
	}

	long FSyncManager.getNTimeouts() {
		return nTimeouts;
	}
	
	void FSyncManager.loadStats(StatsConfig config, EnvironmentStats stats)
			throws DatabaseException {

		stats.setNFSyncs(nFSyncs);
		stats.setNFSyncRequests(nFSyncRequests);
		stats.setNFSyncTimeouts(nTimeouts);

		if (config.getClear()) {
			nFSyncs = 0;
			nFSyncRequests = 0;
			nTimeouts = 0;
		}
	}
	
	


	after (FSyncManager fsm):call(* FSyncManager.hook_timeout()) && target(fsm) {
		synchronized (fsm.fsyncLatch) {
			fsm.nTimeouts++;
		}
	}
	before (FSyncManager fsm):call(* FSyncManager.hook_doFSync(..)) && target(fsm) {
		fsm.nFSyncs++;
	}
	
	before (FSyncManager fsm):call(* FSyncManager.hook_beforeRequest(..)) && target(fsm) {
		fsm.nFSyncRequests++;
	}
	
	/*
	 * fsync stats.
	 */
	public long FileManager.getNFSyncs() {
		return syncManager.getNFSyncs();
	}

	public long FileManager.getNFSyncRequests() {
		return syncManager.getNFSyncRequests();
	}

	public long FileManager.getNFSyncTimeouts() {
		return syncManager.getNTimeouts();
	}

}
