package com.sleepycat.je.incomp;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.incomp.INCompressor.BINSearch;
import com.sleepycat.je.tree.CursorsExistException;
import com.sleepycat.je.tree.NodeNotEmptyException;
import com.sleepycat.je.tree.Tree;

public privileged aspect INCompressorStats {
	// perthis(initINCompressor()) {
	// }
	// pointcut initINCompressor():execution(INCompressor.new(..));

	/* stats */
	private int INCompressor.splitBins = 0;

	private int INCompressor.dbClosedBins = 0;

	private int INCompressor.cursorsBins = 0;

	private int INCompressor.nonEmptyBins = 0;

	private int INCompressor.processedBins = 0;

	/* per-run stats */
	private int INCompressor.splitBinsThisRun = 0;

	private int INCompressor.dbClosedBinsThisRun = 0;

	private int INCompressor.cursorsBinsThisRun = 0;

	private int INCompressor.nonEmptyBinsThisRun = 0;

	private int INCompressor.processedBinsThisRun = 0;

	/**
	 * Reset per-run counters.
	 */
	private void INCompressor.resetPerRunCounters() {
		splitBinsThisRun = 0;
		dbClosedBinsThisRun = 0;
		cursorsBinsThisRun = 0;
		nonEmptyBinsThisRun = 0;
		processedBinsThisRun = 0;
	}

	private void INCompressor.accumulatePerRunCounters() {
		splitBins += splitBinsThisRun;
		dbClosedBins += dbClosedBinsThisRun;
		cursorsBins += cursorsBinsThisRun;
		nonEmptyBins += nonEmptyBinsThisRun;
		processedBins += processedBinsThisRun;
	}

	after(INCompressor inc, BINSearch binSearch) returning(boolean result):
		execution(* INCompressor.findDBAndBIN(..)) && args(binSearch,..) && target(inc) {

		if (result == false) {
			if ((binSearch.db == null) || (binSearch.db.isDeleted())) {
				inc.dbClosedBinsThisRun++;
			} else {
				inc.splitBinsThisRun++;
			}

		}
	}

	void around(INCompressor inc) throws DatabaseException,NodeNotEmptyException, CursorsExistException:
		call(* Tree.delete*(..)) && withincode(* INCompressor.pruneBIN(..)) && this(inc){

		try {
			proceed(inc);
			inc.processedBinsThisRun++;
		} catch (NodeNotEmptyException NNEE) {

			/*
			 * Something was added to the node since the point when the deletion
			 * occurred; we can't prune, and we can throw away this
			 * BINReference.
			 */
			inc.nonEmptyBinsThisRun++;
			throw NNEE;
		} catch (CursorsExistException e) {

			/*
			 * If there are cursors in the way of the delete, retry later. For
			 * example, When we delete a BIN or DBIN, we're guaranteed that
			 * there are no cursors at that node. (otherwise, we wouldn't be
			 * able to remove all the entries. However, there's the possibility
			 * that the BIN that is the parent of the duplicate tree has
			 * resident cursors, and in that case, we would not be able to
			 * remove the whole duplicate tree and DIN root. In that case, we'd
			 * requeue.
			 */
			inc.cursorsBinsThisRun++;
			throw e;
		}

	}

	after(INCompressor inc):
		call(* INCompressor.addBinRefToQueue(..)) && withincode(* INCompressor.compressBin(..)) && this(inc){

		inc.cursorsBinsThisRun++;
	}

	void around(INCompressor inc):
		execution(* INCompressor.hook_internalDoCompress(..)) && this(inc) {
		
		inc.resetPerRunCounters();
		try {
			proceed(inc);
		} finally {
			inc.accumulatePerRunCounters();
		}
	}

	/**
	 * Return stats
	 */
	public void INCompressor.loadStats(StatsConfig config, EnvironmentStats stat)
			throws DatabaseException {

		stat.setSplitBins(splitBins);
		stat.setDbClosedBins(dbClosedBins);
		stat.setCursorsBins(cursorsBins);
		stat.setNonEmptyBins(nonEmptyBins);
		stat.setProcessedBins(processedBins);
		stat.setInCompQueueSize(getBinRefQueueSize());

		if (DEBUG) {
			System.out.println("lazyProcessed = " + lazyProcessed);
			System.out.println("lazyEmpty = " + lazyEmpty);
			System.out.println("lazySplit = " + lazySplit);
			System.out.println("wokenUp=" + wokenUp);
		}

		if (config.getClear()) {
			splitBins = 0;
			dbClosedBins = 0;
			cursorsBins = 0;
			nonEmptyBins = 0;
			processedBins = 0;
			lazyProcessed = 0;
			lazyEmpty = 0;
			lazySplit = 0;
			wokenUp = 0;
		}
	}
}
