package com.sleepycat.je.incomp;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect INCompressorStatsInfrastructure {
	
	
	/* INCompressor */

	/**
	 * The number of bins encountered by the INCompressor that were split
	 * between the time they were put on the compressor queue and when the
	 * compressor ran.
	 */
	private int EnvironmentStats.splitBins;

	/**
	 * The number of bins encountered by the INCompressor that had their
	 * database closed between the time they were put on the compressor queue
	 * and when the compressor ran.
	 */
	private int EnvironmentStats.dbClosedBins;

	/**
	 * The number of bins encountered by the INCompressor that had cursors
	 * referring to them when the compressor ran.
	 */
	private int EnvironmentStats.cursorsBins;

	/**
	 * The number of bins encountered by the INCompressor that were not actually
	 * empty when the compressor ran.
	 */
	private int EnvironmentStats.nonEmptyBins;

	/**
	 * The number of bins that were successfully processed by the IN Compressor.
	 */
	private int EnvironmentStats.processedBins;

	/**
	 * The number of entries in the INCompressor queue when the getStats() call
	 * was made.
	 */
	private int EnvironmentStats.inCompQueueSize;
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getCursorsBins() {
		return cursorsBins;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getDbClosedBins() {
		return dbClosedBins;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getInCompQueueSize() {
		return inCompQueueSize;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getNonEmptyBins() {
		return nonEmptyBins;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getProcessedBins() {
		return processedBins;
	}


	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int EnvironmentStats.getSplitBins() {
		return splitBins;
	}


	
	
	
	

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setCursorsBins(int val) {
		cursorsBins = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setDbClosedBins(int val) {
		dbClosedBins = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setInCompQueueSize(int val) {
		inCompQueueSize = val;
	}


	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNonEmptyBins(int val) {
		nonEmptyBins = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setProcessedBins(int val) {
		processedBins = val;
	}



	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setSplitBins(int val) {
		splitBins = val;
	}

	
	
	after(EnvironmentStats envStats):
		execution(* EnvironmentStats.reset(..)) && target(envStats){
		// InCompressor
		envStats.splitBins = 0;
		envStats.dbClosedBins = 0;
		envStats.cursorsBins = 0;
		envStats.nonEmptyBins = 0;
		envStats.processedBins = 0;
		envStats.inCompQueueSize = 0;
	}
	
	//TODO precedence
	before(StringBuffer sb, EnvironmentStats envStats):
		execution(* EnvironmentStats.hook_fillToStringBuffer(..)) && args(sb) && target(envStats)
	{		
	
		
		sb.append("splitBins=").append(envStats.splitBins).append('\n');
		sb.append("dbClosedBins=").append(envStats.dbClosedBins).append('\n');
		sb.append("cursorsBins=").append(envStats.cursorsBins).append('\n');
		sb.append("nonEmptyBins=").append(envStats.nonEmptyBins).append('\n');
		sb.append("processedBins=").append(envStats.processedBins).append('\n');
		sb.append("inCompQueueSize=").append(envStats.inCompQueueSize).append('\n');

	}
	
	//TODO precedence
	before(StatsConfig config, EnvironmentStats stats, EnvironmentImpl envImpl) throws DatabaseException:
		execution(* EnvironmentImpl.loadStats(StatsConfig, EnvironmentStats)) && this(envImpl) && args(config, stats){
		
		envImpl.inCompressor.loadStats(config, stats);
	}
}
