import com.sleepycat.je.cleaner.Cleaner;
import com.sleepycat.je.cleaner.FileProcessor;
import com.sleepycat.je.log.CleanerFileReader;
import com.sleepycat.je.log.FileReader;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.EnvironmentStats;


public privileged aspect CollectRepeatIteratorReads {
	
	/**
	 * cleaner
	 */
	long Cleaner.nRepeatIteratorReads = 0;

	void around(StatsConfig config, EnvironmentStats stat, Cleaner cleaner):
		execution(* Cleaner.loadStats(..)) && args(config,stat) && this(cleaner) {
		
		stat.setNRepeatIteratorReads(cleaner.nRepeatIteratorReads);
		proceed(config,stat,cleaner);
		if (config.getClear()) {
			cleaner.nRepeatIteratorReads = 0;
		}
	}

	/**
	 * file processor
	 */
	private long FileProcessor.nRepeatIteratorReadsThisRun;
	
	after(FileProcessor fp):
		execution(* FileProcessor.accumulatePerRunCounters()) && this(fp) {
		
		fp.cleaner.nRepeatIteratorReads += fp.nRepeatIteratorReadsThisRun;
	}
	
	after(CleanerFileReader reader, FileProcessor fp):
		call(* FileProcessor.hook_finishedReading(*)) && args(reader) && target(fp) {
		
		fp.nRepeatIteratorReadsThisRun = reader.getNRepeatIteratorReads();
	}
	
	after(FileProcessor fp):
		execution(* FileProcessor.resetPerRunCounters()) && this(fp) {
		
		fp.nRepeatIteratorReadsThisRun=0;
	}

	/**
	 * file reader
	 */
	/*
	 * The number of times we've tried to read in a log entry that was too large
	 * for the read buffer.
	 */
	private long FileReader.nRepeatIteratorReads;
	
	public long FileReader.getNRepeatIteratorReads() {
		return nRepeatIteratorReads;
	}
	
	after(FileReader fr):
		call(* FileReader.hook_markRepeatIteratorRead()) && target(fr) {
		
		fr.nRepeatIteratorReads++;
	}

}
