package com.sleepycat.je;


public aspect LogStatsInfrastructure perthis(call(* EnvironmentStats.*(..))) {
	
	/*
	 * Log activity
	 */

	// turned into singleton fsyncs.
	/*
	 * Number of reads which had to be repeated when faulting in an object from
	 * disk because the read chunk size controlled by je.log.faultReadSize is
	 * too small.
	 */
	private long EnvironmentStats.nRepeatFaultReads;

	/*
	 * Number of times we have to use the temporary marshalling buffer to write
	 * to the log.
	 */
	private long EnvironmentStats.nTempBufferWrites;

	/*
	 * Number of times we try to read a log entry larger than the read buffer
	 * size and can't grow the log buffer to accomodate the large object. This
	 * happens during scans of the log during activities like environment open
	 * or log cleaning. Implies that the the read chunk size controlled by
	 * je.log.iteratorReadSize is too small.
	 */
	private long EnvironmentStats.nRepeatIteratorReads;
	
	after (StringBuffer sb, EnvironmentStats es):
		execution(* hook_fillToStringBuffer(StringBuffer)) && args(sb) && this(es){
		

		sb.append("nRepeatFaultReads=").append(es.nRepeatFaultReads).append('\n');
		sb.append("nTempBufferWrite=").append(es.nTempBufferWrites).append('\n');
		sb.append("nRepeatIteratorReads=").append(es.nRepeatIteratorReads).append(
				'\n');
	}

	after(EnvironmentStats es):call(* EnvironmentStats.reset()) && target(es){
		
		// Log
		es.nRepeatFaultReads = 0;
		es.nTempBufferWrites = 0;
		es.nRepeatIteratorReads = 0;
	}
	

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNRepeatFaultReads() {
		return nRepeatFaultReads;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNTempBufferWrites() {
		return nTempBufferWrites;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public long EnvironmentStats.getNRepeatIteratorReads() {
		return nRepeatIteratorReads;
	}
	
	

	
	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNRepeatFaultReads(long val) {
		nRepeatFaultReads = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNTempBufferWrites(long val) {
		nTempBufferWrites = val;
	}

	/**
	 * Internal use only.
	 */
	public void EnvironmentStats.setNRepeatIteratorReads(long val) {
		nRepeatIteratorReads = val;
	}
}
