package com.sleepycat.je.log;

import com.sleepycat.je.EnvironmentStats;
import com.sleepycat.je.StatsConfig;

public privileged aspect CollectLogManagerStats 
	perthis(loadStats(StatsConfig,EnvironmentStats) || repeatFaultRead()) {

	/*
	 * Number of times we have to repeat a read when we fault in an object
	 * because the initial read was too small.
	 */
	private int nRepeatFaultReads;

	/*
	 * Number of times we have to use the temporary marshalling buffer to write
	 * to the log.
	 */
	private long nTempBufferWrites;
	
	pointcut loadStats(StatsConfig config, EnvironmentStats stats):
		execution(void LogManager.loadStats(StatsConfig, EnvironmentStats)) && args(config,stats);

	before(StatsConfig config, EnvironmentStats stats) : 
		loadStats(config,stats)  {

		stats.setNRepeatFaultReads(nRepeatFaultReads);
		stats.setNTempBufferWrites(nTempBufferWrites);
		if (config.getClear()) {
			nRepeatFaultReads = 0;
			nTempBufferWrites = 0;
		}
	}
	
	pointcut repeatFaultRead():
		call(* LogSource.getBytes(..)) && withincode(* LogManager.getRemainingBuffer(..));

	after():repeatFaultRead(){
		nRepeatFaultReads++;
	}
	
	pointcut tempBufferWrite():
		call(* FileManager.writeLogBuffer(..)) && withincode(* LogManager.logInternal(..));
	
	after():tempBufferWrite(){
		nTempBufferWrites++;
	}
}
