package com.sleepycat.je;

import java.util.Iterator;
import java.util.Map;

import com.sleepycat.je.txn.BasicLocker;
import com.sleepycat.je.txn.Lock;
import com.sleepycat.je.txn.LockInfo;
import com.sleepycat.je.txn.LockManager;
import com.sleepycat.je.txn.Locker;
import com.sleepycat.je.txn.SyncedLockManager;
import com.sleepycat.je.txn.Txn;
import com.sleepycat.je.txn.TxnManager;

public privileged aspect CollectStatsAspect {
	
	/**
	 * Get lock count, for per transaction lock stats, for internal debugging.
	 */
	public abstract LockStats Locker.collectStats(LockStats stats)
			throws DatabaseException;

	
	
	/**
	 * stats
	 */
	public LockStats BasicLocker.collectStats(LockStats stats) throws DatabaseException {

		if (ownedLock != null) {
			if (ownedLock.isOwnedWriteLock(this)) {
				stats.setNWriteLocks(stats.getNWriteLocks() + 1);
			} else {
				stats.setNReadLocks(stats.getNReadLocks() + 1);
			}
		}
		if (ownedLockSet != null) {
			Iterator iter = ownedLockSet.iterator();

			while (iter.hasNext()) {
				Lock l = (Lock) iter.next();
				if (l.isOwnedWriteLock(this)) {
					stats.setNWriteLocks(stats.getNWriteLocks() + 1);
				} else {
					stats.setNReadLocks(stats.getNReadLocks() + 1);
				}
			}
		}
		return stats;
	}
	

	/**
	 * Collect lock related stats.
	 */
	public LockStats TxnManager.lockStat(StatsConfig config) throws DatabaseException {

		return lockManager.lockStat(config);
	}
	
	
	
	
	/**
	 * Statistics
	 */
	public LockStats LockManager.lockStat(StatsConfig config) throws DatabaseException {

		LockStats stats = new LockStats();
		stats.setNRequests(nRequests);
		stats.setNWaits(nWaits);
		if (config.getClear()) {
			nWaits = 0;
			nRequests = 0;
		}

		hook_lockStatLatches(stats);
	

		/* Dump info about the lock table. */
		if (!config.getFast()) {
			dumpLockTable(stats);
		}
		return stats;
	}

	private void LockManager.hook_lockStatLatches(LockStats stats) {
	}

	/**
	 * Dump the lock table to the lock stats.
	 */
	//TODO needed to make public
	abstract public void LockManager.dumpLockTable(LockStats stats)
			throws DatabaseException;

	/**
	 * Do the real work of dumpLockTableInternal.
	 */
	//TODO needed to make public
	public void LockManager.dumpLockTableInternal(LockStats stats, int i) {
		Map lockTable = lockTables[i];
		stats.accumulateNTotalLocks(lockTable.size());
		Iterator iter = lockTable.values().iterator();
		while (iter.hasNext()) {
			Lock lock = (Lock) iter.next();
			stats.setNWaiters(stats.getNWaiters() + lock.nWaiters());
			stats.setNOwners(stats.getNOwners() + lock.nOwners());

			/* Go through all the owners for a lock. */
			Iterator ownerIter = lock.getOwnersClone().iterator();
			while (ownerIter.hasNext()) {
				LockInfo info = (LockInfo) ownerIter.next();
				if (info.getLockType().isWriteLock()) {
					stats.setNWriteLocks(stats.getNWriteLocks() + 1);
				} else {
					stats.setNReadLocks(stats.getNReadLocks() + 1);
				}
			}
		}
	}
	
	/**
	 * @see LockManager#dumpLockTable
	 */
	//TODO needed to make public
	public void SyncedLockManager.dumpLockTable(LockStats stats) throws DatabaseException {

		for (int i = 0; i < nLockTables; i++) {
			synchronized (lockTables[i]) {
				dumpLockTableInternal(stats, i);
			}
		}
	}
	
	
	/**
	 * stats
	 */
	public LockStats Txn.collectStats(LockStats stats) throws DatabaseException {

		synchronized (this) {
			int nReadLocks = (readLocks == null) ? 0 : readLocks.size();
			stats.setNReadLocks(stats.getNReadLocks() + nReadLocks);
			int nWriteLocks = (writeInfo == null) ? 0 : writeInfo.size();
			stats.setNWriteLocks(stats.getNWriteLocks() + nWriteLocks);
		}

		return stats;
	}
}
