package com.sleepycat.je;

public privileged aspect SequenceStatsCollector {
	
	private int Sequence.nGets;

	private int Sequence.nCachedGets;
	
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public SequenceStats Sequence.getStats(StatsConfig config) throws DatabaseException {

		if (config == null) {
			config = StatsConfig.DEFAULT;
		}

		if (!config.getFast()) {

			/*
			 * storedValue may have been updated by another handle since it was
			 * last read by this handle. Fetch the last written value.
			 * READ_UNCOMMITTED must be used to avoid lock conflicts.
			 */
			Cursor cursor = db.openCursor(OperationContext.NULL, null);
			try {
				readDataRequired(cursor, LockMode.READ_UNCOMMITTED);
			} finally {
				cursor.close();
			}
		}

		SequenceStats stats = new SequenceStats(nGets, nCachedGets,
				storedValue, cacheValue, cacheLast, rangeMin, rangeMax,
				cacheSize);

		if (config.getClear()) {
			nGets = 0;
			nCachedGets = 0;
		}

		return stats;
	}
	
	after (boolean cached, Sequence sequence):
		call(* Sequence.hook_getDone(..)) && args(cached) && target(sequence){
		
		/* Increment stats. */
		sequence.nGets += 1;
		if (cached) {
			sequence.nCachedGets += 1;
		}
	}
}
