package com.sleepycat.je;

import java.util.Iterator;

import javax.transaction.xa.Xid;

import com.sleepycat.je.txn.Locker;
import com.sleepycat.je.txn.Txn;
import com.sleepycat.je.txn.TxnManager;

public privileged aspect TransactionStatsCollector {
	/* Locker Stats */
	private int TxnManager.numCommits=0;

	private int TxnManager.numAborts=0;

	private int TxnManager.numXAPrepares=0;

	private int TxnManager.numXACommits=0;

	private int TxnManager.numXAAborts=0;
	
	after(boolean isCommit,TxnManager txnm):
		((execution(* TxnManager.unRegisterTxn(Txn, boolean)) && args(*,isCommit)) ||
		(execution(* TxnManager.unRegisterXATxn(Xid, boolean)) && args(*,isCommit))) &&
		target(txnm){
		
		if (isCommit) {
			txnm.numXACommits++;
		} else {
			txnm.numXAAborts++;
		}
	}
		
	after(boolean isPrepare,TxnManager txnm):
		execution(* TxnManager.registerXATxn(..)) && args(*,*,isPrepare) &&
		target(txnm){
		
		if (isPrepare) {
			txnm.numXAPrepares++;
		}
	}
	
	/**
	 * Collect transaction related stats.
	 */
	public TransactionStats TxnManager.txnStat(StatsConfig config)
			throws DatabaseException {

		TransactionStats stats = new TransactionStats();
			stats.setNCommits(numCommits);
			stats.setNAborts(numAborts);
			stats.setNXAPrepares(numXAPrepares);
			stats.setNXACommits(numXACommits);
			stats.setNXAAborts(numXAAborts);
			stats.setNActive(allTxns.size());
			TransactionStats.Active[] activeSet = new TransactionStats.Active[stats.getNActive()];
			stats.setActiveTxns(activeSet);
			Iterator iter = allTxns.iterator();
			int i = 0;
			while (iter.hasNext()) {
				Locker txn = (Locker) iter.next();
				activeSet[i] = new TransactionStats.Active(txn.toString(), txn
						.getId(), 0);
				i++;
			}
			if (config.getClear()) {
				numCommits = 0;
				numAborts = 0;
				numXACommits = 0;
				numXAAborts = 0;
			}
		return stats;
	}
}
